@@ -1,4 +1,4 @@
-/* 
+/*
    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Bastien Nocera
    Copyright (C) 2003, 2004 Colin Walters <walters@rhythmbox.org>
 
@@ -38,7 +38,7 @@
 #ifndef TOTEM_PL_PARSER_MINI
 gboolean
 totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
-			   TotemPlParserIterFunc func, 
+			   TotemPlParserIterFunc func,
 			   GFile *output, const char *title,
 			   gpointer user_data, GError **error)
 {
@@ -123,7 +123,7 @@ totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
 }
 
 static char *
-ensure_utf8_valid (char *input) 
+ensure_utf8_valid (char *input)
 {
 	char *utf8_valid;
 
@@ -147,7 +147,8 @@ TotemPlParserResult
 totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 				       GFile *file,
 				       GFile *_base_file,
-				       const char *contents)
+				       const char *contents,
+				       TotemPlParseData *parse_data)
 {
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	GFile *base_file;
@@ -155,11 +156,15 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 	int i, num_entries;
 	char *playlist_title;
 	gboolean fallback;
+	GHashTable *entries;
+
+	entries = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
 	lines = g_strsplit_set (contents, "\r\n", 0);
 
 	/* [playlist] */
 	i = 0;
+	num_entries = 0;
 	playlist_title = NULL;
 
 	/* Ignore empty lines */
@@ -167,13 +172,14 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 		i++;
 
 	if (lines[i] == NULL
-			|| g_ascii_strncasecmp (lines[i], "[playlist]",
-				(gsize)strlen ("[playlist]")) != 0) {
+	    || g_ascii_strncasecmp (lines[i], "[playlist]",
+				    (gsize)strlen ("[playlist]")) != 0) {
+		g_strfreev (lines);
 		goto bail;
 	}
 
 	playlist_title = totem_pl_parser_read_ini_line_string (lines,
-			"X-GNOME-Title");
+							       "X-GNOME-Title");
 
 	if (playlist_title != NULL) {
 		totem_pl_parser_add_uri (parser,
@@ -183,23 +189,36 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 					 NULL);
 	}
 
-	/* numberofentries=? */
-	num_entries = totem_pl_parser_read_ini_line_int (lines, "numberofentries");
+	/* Load the file in hash table to speed up the later processing */
+	for (i = 0; lines[i] != NULL; i++) {
+		char **bits;
+		char *value;
+		gint len;
 
-	if (num_entries == -1) {
-		num_entries = 0;
+		if (totem_pl_parser_line_is_empty (lines[i]))
+			continue;
 
-		for (i = 0; lines[i] != NULL; i++) {
-			if (totem_pl_parser_line_is_empty (lines[i]))
-				continue;
+		if (lines[i][0] == '#' || lines[i][0] == '[')
+			continue;
 
-			if (g_ascii_strncasecmp (g_strchug (lines[i]), "file", (gsize)strlen ("file")) == 0)
-				num_entries++;
+		bits = g_strsplit (lines[i], "=", 2);
+		if (bits[0] == NULL || bits [1] == NULL) {
+			g_strfreev (bits);
+			continue;
 		}
 
-		if (num_entries == 0)
-			goto bail;
+		if (g_ascii_strncasecmp (g_strchug (bits[0]), "file", strlen ("file")) == 0)
+			num_entries++;
+
+		len = strlen (bits[1]);
+		value = g_strdup (bits[1]);
+
+		g_hash_table_insert (entries,
+				     g_ascii_strdown (bits[0], strlen (bits[0])),
+				     value);
+		g_strfreev (bits);
 	}
+	g_strfreev (lines);
 
 	/* Base? */
 	if (_base_file == NULL)
@@ -221,37 +240,33 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 		/* Genre is our own little extension */
 		genre_key = g_strdup_printf ("genre%d", i);
 
-		file_str = totem_pl_parser_read_ini_line_string (lines, (const char*)file_key);
-		title = totem_pl_parser_read_ini_line_string (lines, (const char*)title_key);
-		genre = totem_pl_parser_read_ini_line_string (lines, (const char*)genre_key);
-		length = totem_pl_parser_read_ini_line_string (lines, (const char*)length_key);
+		file_str = g_hash_table_lookup (entries, file_key);
+		title = g_hash_table_lookup (entries, title_key);
+		genre = g_hash_table_lookup (entries, genre_key);
+		length = g_hash_table_lookup (entries, length_key);
 
 		g_free (file_key);
 		g_free (title_key);
 		g_free (genre_key);
 		g_free (length_key);
 
-		if (file_str == NULL) {
-			g_free (title);
-			g_free (genre);
-			g_free (length);
+		if (file_str == NULL)
 			continue;
-		}
 
-		fallback = parser->priv->fallback;
-		if (parser->priv->recurse)
-			parser->priv->fallback = FALSE;
+		fallback = parse_data->fallback;
+		if (parse_data->recurse)
+			parse_data->fallback = FALSE;
 
 		/* Get the length, if it's negative, that means that we have a stream
 		 * and should push the entry straight away */
 		if (length != NULL)
-			length_num = totem_pl_parser_parse_duration (length, parser->priv->debug);
+			length_num = totem_pl_parser_parse_duration (length, totem_pl_parser_is_debugging_enabled (parser));
 
 		if (strstr (file_str, "://") != NULL || file_str[0] == G_DIR_SEPARATOR) {
 			GFile *target;
 
 			target = g_file_new_for_commandline_arg (file_str);
-			if (length_num < 0 || totem_pl_parser_parse_internal (parser, target, NULL) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+			if (length_num < 0 || totem_pl_parser_parse_internal (parser, target, NULL, parse_data) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 				totem_pl_parser_add_uri (parser,
 							 TOTEM_PL_PARSER_FIELD_URI, file_str,
 							 TOTEM_PL_PARSER_FIELD_TITLE, title,
@@ -268,7 +283,7 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 			target = g_file_get_child_for_display_name (base_file, utf8_filename, NULL);
 			g_free (utf8_filename);
 
-			if (length_num < 0 || totem_pl_parser_parse_internal (parser, target, base_file) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+			if (length_num < 0 || totem_pl_parser_parse_internal (parser, target, base_file, parse_data) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 				totem_pl_parser_add_uri (parser,
 							 TOTEM_PL_PARSER_FIELD_FILE, target,
 							 TOTEM_PL_PARSER_FIELD_TITLE, title,
@@ -280,11 +295,7 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 			g_object_unref (target);
 		}
 
-		parser->priv->fallback = fallback;
-		g_free (file_str);
-		g_free (title);
-		g_free (genre);
-		g_free (length);
+		parse_data->fallback = fallback;
 	}
 
 	if (playlist_title != NULL)
@@ -294,7 +305,7 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
 
 bail:
 	g_free (playlist_title);
-	g_strfreev (lines);
+        g_hash_table_destroy (entries);
 
 	return retval;
 }
@@ -303,6 +314,7 @@ TotemPlParserResult
 totem_pl_parser_add_pls (TotemPlParser *parser,
 			 GFile *file,
 			 GFile *base_file,
+			 TotemPlParseData *parse_data,
 			 gpointer data)
 {
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
@@ -317,7 +329,7 @@ totem_pl_parser_add_pls (TotemPlParser *parser,
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
-	retval = totem_pl_parser_add_pls_with_contents (parser, file, base_file, contents);
+	retval = totem_pl_parser_add_pls_with_contents (parser, file, base_file, contents, parse_data);
 	g_free (contents);
 
 	return retval;
